@@ -239,19 +239,26 @@ impl PTYImpl for WinPTY {
         self.process.write(buf)
     }
 
-    fn is_eof(&mut self) -> Result<bool, OsString> {
+    fn is_eof(&self) -> Result<bool, OsString> {
         self.process.is_eof()
     }
 
-    fn get_exitstatus(&mut self) -> Result<Option<u32>, OsString> {
+    fn get_exitstatus(&self) -> Result<Option<u32>, OsString> {
         self.process.get_exitstatus()
     }
 
-    fn is_alive(&mut self) -> Result<bool, OsString> {
+    fn is_alive(&self) -> Result<bool, OsString> {
         self.process.is_alive()
     }
 
     fn get_pid(&self) -> u32 {
         self.process.get_pid()
     }
+
+    fn get_fd(&self) -> isize {
+        self.process.get_fd()
+    }
 }
+
+unsafe impl Send for WinPTY {}
+unsafe impl Sync for WinPTY {}
