@@ -39,7 +39,13 @@ fn read_write_conpty() {
     let mut pty = PTY::new_with_backend(&pty_args, PTYBackend::ConPTY).unwrap();
     pty.spawn(appname, None, None, None).unwrap();
 
-    let regex = Regex::new(r".*Microsoft Windows.*").unwrap();
+    let mut re_pattern: &str = r".*Microsoft Windows.*";
+
+    if env::var_os("CI").is_some() {
+        re_pattern = ".*cmd.*"
+    }
+
+    let regex = Regex::new(re_pattern).unwrap();
     let mut output_str = "";
     let mut out: OsString;
 
@@ -80,10 +86,6 @@ fn read_write_conpty() {
 
 #[test]
 fn set_size_conpty() {
-    if &env::var("CI").unwrap_or("0".to_owned()) == "1" {
-        return;
-    }
-
     let pty_args = PTYArgs {
         cols: 80,
         rows: 25,
@@ -123,6 +125,14 @@ fn set_size_conpty() {
     assert_eq!(cols, pty_args.cols);
 
     pty.set_size(90, 30).unwrap();
+
+    if &env::var("CI").unwrap_or("0".to_owned()) == "1" {
+        return;
+    }
+
+    pty.write("cls\r\n".into()).unwrap();
+    pty.write("cls\r\n".into()).unwrap();
+    pty.write("cls\r\n".into()).unwrap();
     pty.write("cls\r\n".into()).unwrap();
 
     pty.write("powershell -command \"&{(get-host).ui.rawui.WindowSize;}\"\r\n".into()).unwrap();
@@ -135,6 +145,8 @@ fn set_size_conpty() {
         output_str = out.to_str().unwrap();
     }
 
+    println!("{:?}", output_str);
+
     let parts: Vec<&str> = output_str.split("\r\n").collect();
     let num_regex = Regex::new(r"\s+(\d+)\s+(\d+).*").unwrap();
     for part in parts {
